
/*
 * This program source code file is part of SystemK, a library in the KTag project.
 *
 * 🛡️ <https://ktag.clubk.club> 🃞
 *
 * Copyright © 2016-2025 Joseph P. Kearney and the KTag developers.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
 * details.
 *
 * There should be a copy of the GNU Affero General Public License in the LICENSE
 * file in the root of this repository.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "SystemK.h"

static color_t RSSSIColor = (color_t) 0x00000000;
static uint8_t Brightness = 255;

static void Reset(void * Data)
{
    RSSSIColor = *((color_t *)Data);
    Brightness = 255 + *((int8_t *)Data) + *((int8_t *)Data);
}

static AnimationStepResult_T NextStep(void)
{
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
    Set_Barrel_Flash(Color(0x00, 0x00, 0x00, 0x00));
    Set_Heart(ApplyMask(RSSSIColor, Brightness));
    Set_Square(ApplyMask(RSSSIColor, Brightness));
    Set_Circle(ApplyMask(RSSSIColor, Brightness));
    Set_Arrow(ApplyMask(RSSSIColor, Brightness));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
    NeoPixels_Set_Color_Range_On_All_Channels(0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL-1, ApplyMask(RSSSIColor, Brightness));
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS

    return ANIMATION_ONGOING;
}

Animation_T BLE_RSSI_Animation = 
{
    .Reset = Reset,
    .NextStep = NextStep
};
