
/*
 * This program source code file is part of SystemK, a library in the KTag project.
 *
 * 🛡️ <https://ktag.clubk.club> 🃞
 *
 * Copyright © 2016-2025 Joseph P. Kearney and the KTag developers.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
 * details.
 *
 * There should be a copy of the GNU Affero General Public License in the LICENSE
 * file in the root of this repository.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "SystemK.h"

static uint8_t N_Lights;
static color_t My_Color = COLOR_BLACK;

static const uint8_t BRIGHTNESS = 0x7F;

static void Reset(void *Data)
{
    N_Lights = *((uint8_t *)Data);
    My_Color = HW_NeoPixels_Get_My_Color();
}

static AnimationStepResult_T NextStep(void)
{
    switch (N_Lights)
    {
    case 0:
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
        Set_Barrel_Flash(ApplyMask(My_Color, 0x00));
        Set_Heart(ApplyMask(My_Color, 0x00));
        Set_Square(ApplyMask(My_Color, 0x00));
        Set_Circle(ApplyMask(My_Color, 0x00));
        Set_Arrow(ApplyMask(My_Color, 0x00));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_RECEIVER, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_DISPLAY, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_EFFECTS, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS
        break;

    case 1:
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
        Set_Barrel_Flash(ApplyMask(My_Color, 0x00));
        Set_Heart(ApplyMask(My_Color, 0x00));
        Set_Square(ApplyMask(My_Color, 0x00));
        Set_Circle(ApplyMask(My_Color, 0x00));
        Set_Arrow(ApplyMask(My_Color, BRIGHTNESS));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, RIGHT_RECEIVER_INDICATOR_PIXEL, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, LEFT_RECEIVER_INDICATOR_PIXEL, COLOR_BLACK);

        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_DISPLAY, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 4, ApplyMask(My_Color, BRIGHTNESS));

        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_EFFECTS, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 4, ApplyMask(My_Color, BRIGHTNESS));
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS
        break;

    case 2:
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
        Set_Barrel_Flash(ApplyMask(My_Color, 0x00));
        Set_Heart(ApplyMask(My_Color, 0x00));
        Set_Square(ApplyMask(My_Color, 0x00));
        Set_Circle(ApplyMask(My_Color, BRIGHTNESS));
        Set_Arrow(ApplyMask(My_Color, BRIGHTNESS));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, RIGHT_RECEIVER_INDICATOR_PIXEL, COLOR_BLACK);
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, LEFT_RECEIVER_INDICATOR_PIXEL, ApplyMask(My_Color, BRIGHTNESS));

        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_DISPLAY, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 3, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 4, ApplyMask(My_Color, BRIGHTNESS));

        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_EFFECTS, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 3, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 4, ApplyMask(My_Color, BRIGHTNESS));
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS
        break;

    case 3:
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
        Set_Barrel_Flash(ApplyMask(My_Color, 0x00));
        Set_Heart(ApplyMask(My_Color, 0x00));
        Set_Square(ApplyMask(My_Color, BRIGHTNESS));
        Set_Circle(ApplyMask(My_Color, BRIGHTNESS));
        Set_Arrow(ApplyMask(My_Color, BRIGHTNESS));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, RIGHT_RECEIVER_INDICATOR_PIXEL, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, LEFT_RECEIVER_INDICATOR_PIXEL, COLOR_BLACK);

        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_DISPLAY, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 2, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 3, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 4, ApplyMask(My_Color, BRIGHTNESS));

        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_EFFECTS, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 2, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 3, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 4, ApplyMask(My_Color, BRIGHTNESS));
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS
        break;

    case 4:
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
        Set_Barrel_Flash(ApplyMask(My_Color, 0x00));
        Set_Heart(ApplyMask(My_Color, BRIGHTNESS));
        Set_Square(ApplyMask(My_Color, BRIGHTNESS));
        Set_Circle(ApplyMask(My_Color, BRIGHTNESS));
        Set_Arrow(ApplyMask(My_Color, BRIGHTNESS));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, RIGHT_RECEIVER_INDICATOR_PIXEL, COLOR_BLACK);
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_RECEIVER, LEFT_RECEIVER_INDICATOR_PIXEL, ApplyMask(My_Color, BRIGHTNESS));
        
        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_DISPLAY, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 1, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 2, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 3, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_DISPLAY, 4, ApplyMask(My_Color, BRIGHTNESS));

        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_EFFECTS, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, 0x00));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 1, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 2, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 3, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_EFFECTS, 4, ApplyMask(My_Color, BRIGHTNESS));
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS
        break;

    case 5:
    default:
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
        Set_Barrel_Flash(ApplyMask(My_Color, BRIGHTNESS));
        Set_Heart(ApplyMask(My_Color, BRIGHTNESS));
        Set_Square(ApplyMask(My_Color, BRIGHTNESS));
        Set_Circle(ApplyMask(My_Color, BRIGHTNESS));
        Set_Arrow(ApplyMask(My_Color, BRIGHTNESS));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_RECEIVER, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_DISPLAY, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, BRIGHTNESS));
        NeoPixels_Set_Color_Range(NEOPIXEL_CHANNEL_EFFECTS, 0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, ApplyMask(My_Color, BRIGHTNESS));
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS
        break;
    }

    return ANIMATION_ONGOING;
}

Animation_T Countdown_Animation =
    {
        .Reset = Reset,
        .NextStep = NextStep
    };
