
/*
 * This program source code file is part of SystemK, a library in the KTag project.
 *
 * 🛡️ <https://ktag.clubk.club> 🃞
 *
 * Copyright © 2016-2025 Joseph P. Kearney and the KTag developers.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
 * details.
 *
 * There should be a copy of the GNU Affero General Public License in the LICENSE
 * file in the root of this repository.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "SystemK.h"

static uint16_t Time_in_Animation_in_ms = 0;
static color_t Flash_Color = COLOR_BLACK;

static void Reset(void * Data)
{
    Time_in_Animation_in_ms = 0;
    Flash_Color = HW_NeoPixels_Get_My_Color();
    NeoPixels_Set_Color_Range_On_All_Channels(0, CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL - 1, COLOR_BLACK);
}

static AnimationStepResult_T NextStep(void)
{
    AnimationStepResult_T result;

    if (Time_in_Animation_in_ms < 250)
    {
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_BARREL, BARREL_FLASH_PIXEL, Flash_Color);

        Time_in_Animation_in_ms += CONFIG_KTAG_ANIMATION_STEP_TIME_IN_ms;
        result = ANIMATION_ONGOING;
    }
    else
    {
        NeoPixels_Set_Color(NEOPIXEL_CHANNEL_BARREL, BARREL_FLASH_PIXEL, COLOR_BLACK);
        result = ANIMATION_COMPLETE;
    }

    return result;
}

Animation_T Shot_Fired_Animation = 
{
    .Reset = Reset,
    .NextStep = NextStep
};
