
/*
 * This program source code file is part of SystemK, a library in the KTag project.
 *
 * 🛡️ <https://ktag.clubk.club> 🃞
 *
 * Copyright © 2016-2025 Joseph P. Kearney and the KTag developers.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
 * details.
 *
 * There should be a copy of the GNU Affero General Public License in the LICENSE
 * file in the root of this repository.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "SystemK.h"

static uint8_t Fade = 0xFF;
static color_t My_Color = COLOR_BLACK;

static void Reset(void * Data)
{
    Fade = 0xFF;
    My_Color = HW_NeoPixels_Get_My_Color();
}

static AnimationStepResult_T NextStep(void)
{
    uint8_t phase_0 = Fade;
    uint8_t phase_1 = phase_0 - (uint8_t)64;
    uint8_t phase_2 = phase_1 - (uint8_t)64;
    uint8_t phase_3 = phase_2 - (uint8_t)64;
    
#if (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 1)
    Set_Barrel_Flash(Color(Sine8[phase_0], 0xFF, 0x00, 0x00));
    Set_Heart(Color(Sine8[phase_1], 0xFF, 0x00, 0x00));
    Set_Square(Color(Sine8[phase_2], 0xFF, 0x00, 0x00));
    Set_Circle(Color(Sine8[phase_3], 0xFF, 0x00, 0x00));
    Set_Arrow(Color(Sine8[phase_0], 0xFF, 0x00, 0x00));
#elif (CONFIG_KTAG_N_NEOPIXEL_CHANNELS == 4)
    for (uint_fast8_t pixel = 0; pixel < CONFIG_KTAG_MAX_NEOPIXELS_PER_CHANNEL; pixel++)
    {
        switch (pixel % 4)
        {
            default:
            case 0:
                NeoPixels_Set_Color_On_All_Channels(pixel, ApplyMask(My_Color, Sine8[phase_0]));
                break;
                
            case 1:
                NeoPixels_Set_Color_On_All_Channels(pixel, ApplyMask(My_Color, Sine8[phase_1]));
                break;
                
            case 2:
                NeoPixels_Set_Color_On_All_Channels(pixel, ApplyMask(My_Color, Sine8[phase_2]));
                break;
                
            case 3:
                NeoPixels_Set_Color_On_All_Channels(pixel, ApplyMask(My_Color, Sine8[phase_3]));
                break;
        }
    }
    NeoPixels_Set_Color(NEOPIXEL_CHANNEL_BARREL, BARREL_FLASH_PIXEL, COLOR_WHITE);
#endif // CONFIG_KTAG_N_NEOPIXEL_CHANNELS
    
    Fade += 2;

    return ANIMATION_ONGOING;
}

Animation_T Tagged_Out_Animation = 
{
    .Reset = Reset,
    .NextStep = NextStep
};
